<?php

namespace Drupal\Tests\social_auth_entra_id\Unit\Controller;

use Drupal\Core\Config\ConfigFactoryInterface;
use Drupal\Core\Language\LanguageManagerInterface;
use Drupal\Core\Logger\LoggerChannelFactoryInterface;
use Drupal\Core\Messenger\MessengerInterface;
use Drupal\Core\Routing\TrustedRedirectResponse;
use Drupal\social_auth_entra_id\Controller\SocialAuthEntraIdController;
use Drupal\Tests\UnitTestCase;
use GuzzleHttp\ClientInterface;
use Psr\Http\Message\ResponseInterface;
use Symfony\Component\HttpFoundation\RedirectResponse;
use Symfony\Component\HttpFoundation\Request;

/**
 * Unit tests for the SocialAuthEntraIdController class.
 *
 * @group social_auth_entra_id
 */
class SocialAuthEntraIdControllerTest extends UnitTestCase {

  /**
   * The controller under test.
   *
   * @var \Drupal\social_auth_entra_id\Controller\SocialAuthEntraIdController
   */
  protected $controller;

  /**
   * Mocks for dependencies.
   */
  protected $configFactory;
  protected $httpClient;
  protected $messenger;
  protected $languageManager;
  protected $loggerFactory;

  /**
   * {@inheritdoc}
   */
  protected function setUp(): void {
    parent::setUp();

    // Mock dependencies.
    $this->configFactory = $this->createMock(ConfigFactoryInterface::class);
    $this->httpClient = $this->createMock(ClientInterface::class);
    $this->messenger = $this->createMock(MessengerInterface::class);
    $this->languageManager = $this->createMock(LanguageManagerInterface::class);
    $this->loggerFactory = $this->createMock(LoggerChannelFactoryInterface::class);

    // Initialize the controller with mocked dependencies.
    $this->controller = new SocialAuthEntraIdController(
      $this->configFactory,
      $this->httpClient,
      $this->messenger,
      $this->languageManager,
      $this->loggerFactory
    );
  }

  /**
   * Tests redirectToMicrosoft() method for a successful redirect.
   */
  public function testRedirectToMicrosoft() {
    // Implement assertions to validate the redirect behavior.
    $result = $this->controller->redirectToMicrosoft();
    $this->assertNotNull($result, 'Expected a redirect response.');
    // Additional assertions as needed.
    $config = $this->createMock(ConfigFactoryInterface::class);
    $config->expects($this->exactly(2))
      ->method('get')
      ->willReturnMap([
        ['social_auth_entra_id.settings', 'client_id', 'test_client_id'],
        ['social_auth_entra_id.settings', 'tenant_id', 'test_tenant_id'],
      ]);

    $this->configFactory->expects($this->once())
      ->method('get')
      ->with('social_auth_entra_id.settings')
      ->willReturn($config);

    $redirect_response = $this->controller->redirectToMicrosoft();
    $this->assertInstanceOf(TrustedRedirectResponse::class, $redirect_response);
    $this->assertStringContainsString('https://login.microsoftonline.com/test_tenant_id/oauth2/v2.0/authorize', $redirect_response->getTargetUrl());
  }

  /**
   * Tests redirectToMicrosoft() method with missing client ID.
   */
  public function testRedirectToMicrosoftMissingClientId() {
    $config = $this->createMock(ConfigFactoryInterface::class);
    $config->method('get')
      ->willReturnMap([
        ['social_auth_entra_id.settings', 'client_id', NULL],
        ['social_auth_entra_id.settings', 'tenant_id', 'test_tenant_id'],
      ]);

    $this->configFactory->expects($this->once())
      ->method('get')
      ->with('social_auth_entra_id.settings')
      ->willReturn($config);

    $this->messenger->expects($this->once())
      ->method('addError')
      ->with($this->stringContains('Client ID is missing.'));

    $redirect_response = $this->controller->redirectToMicrosoft();
    $this->assertInstanceOf(RedirectResponse::class, $redirect_response);
  }

  /**
   * Tests handleMicrosoftCallback() method with valid authorization code.
   */
  public function testHandleMicrosoftCallbackValidCode() {
    $config = $this->createMock(ConfigFactoryInterface::class);
    $config->method('get')
      ->willReturnMap([
        ['social_auth_entra_id.settings', 'client_id', 'test_client_id'],
        ['social_auth_entra_id.settings', 'client_secret', 'test_client_secret'],
        ['social_auth_entra_id.settings', 'tenant_id', 'test_tenant_id'],
        ['social_auth_entra_id.settings', 'login_behavior', 'register_and_login'],
      ]);

    $this->configFactory->expects($this->once())
      ->method('get')
      ->with('social_auth_entra_id.settings')
      ->willReturn($config);

    $this->httpClient->expects($this->once())
      ->method('post')
      ->willReturn($this->createMockResponse(['access_token' => 'mock_access_token']));

    $this->httpClient->expects($this->once())
      ->method('get')
      ->willReturn($this->createMockResponse(['mail' => 'user@example.com']));

    $request = new Request([], [], ['code' => 'test_code']);
    $redirect_response = $this->controller->handleMicrosoftCallback($request);

    $this->assertInstanceOf(RedirectResponse::class, $redirect_response);
  }

  /**
   * Tests handleMicrosoftCallback() method with missing authorization code.
   */
  public function testHandleMicrosoftCallbackMissingCode() {
    $this->messenger->expects($this->once())
      ->method('addError')
      ->with($this->stringContains('Authorization code is missing.'));

    $request = new Request();
    $redirect_response = $this->controller->handleMicrosoftCallback($request);

    $this->assertInstanceOf(RedirectResponse::class, $redirect_response);
  }

  /**
   * Helper function to create mock response.
   */
  protected function createMockResponse(array $data) {
    $response = $this->createMock(ResponseInterface::class);
    $response->method('getBody')
      ->willReturn(json_encode($data));
    return $response;
  }

}
